from tacotoolbox.specification.datamodel_contacts import Contact
from tacotoolbox.specification.datamodel_extent import Extent
from tacotoolbox.specification.datamodel_hyperlink import HyperLink
from tacotoolbox.specification.datamodel_labels import Labels
from tacotoolbox.specification.datamodel_opticaldata import OpticalData
from tacotoolbox.specification.datamodel_scientific import Scientific
from tacotoolbox.specification.datamodel_split import SplitStrategy
from tacotoolbox.specification.datamodel_tasks import Task
